% csolve  Solves a custom quadratic program very rapidly.
%
% [vars, status] = csolve(params, settings)
%
% solves the convex optimization problem
%
%   minimize(norm(B*delta - u, 1) + alpha*(norm(D(1)*delta(1), 1) + norm(D(2)*delta(2), 1) + norm(D(3)*delta(3), 1) + norm(D(4)*delta(4), 1) + norm(D(5)*delta(5), 1) + norm(D(6)*delta(6), 1)))
%   subject to
%     deltamin(1) <= delta(1)
%     delta(1) <= deltamax(1)
%     deltamin(2) <= delta(2)
%     delta(2) <= deltamax(2)
%     deltamin(3) <= delta(3)
%     delta(3) <= deltamax(3)
%     deltamin(4) <= delta(4)
%     delta(4) <= deltamax(4)
%     deltamin(5) <= delta(5)
%     delta(5) <= deltamax(5)
%     deltamin(6) <= delta(6)
%     delta(6) <= deltamax(6)
%
% with variables
%    delta   6 x 1
%
% and parameters
%        B   3 x 6
%        D   1 x 6
%    alpha   1 x 1    positive
% deltamax   6 x 1    positive
% deltamin   6 x 1    negative
%        u   3 x 1
%
% Note:
%   - Check status.converged, which will be 1 if optimization succeeded.
%   - You don't have to specify settings if you don't want to.
%   - To hide output, use settings.verbose = 0.
%   - To change iterations, use settings.max_iters = 20.
%   - You may wish to compare with cvxsolve to check the solver is correct.
%
% Specify params.B, ..., params.u, then run
%   [vars, status] = csolve(params, settings)


% Produced by CVXGEN, 2011-05-06 06:09:04 -0700.
% CVXGEN is Copyright (C) 2006-2011 Jacob Mattingley, jem@cvxgen.com.
% The code in this file is Copyright (C) 2006-2011 Jacob Mattingley.
% CVXGEN, or solvers produced by CVXGEN, cannot be used for commercial
% applications without prior written permission from Jacob Mattingley.

% Filename: csolve.m.
% Description: Help file for the Matlab solver interface.
